/**
 * @file
 * ACR122 API library header file.
 * @version 1.0
 * @date    8 Nov 2010
 * @warning Copyright (C) 2009-2010 Advanced Card Systems Ltd. All rights reserved.
 */
 
/**
 * @mainpage
 *
 * @section Introduction Introduction
 *
 * This documentation covers the APIs provided by ACR122 API library. The
 * library supports ACR122S and ACR122L reader. Note that all functions related
 * to LCD operation and Slot 1 and 2 are for ACR122L only. ACR122S cannot use
 * these functions.
 *
 * @section UsingACR122API Using ACR122 API
 *
 * To use ACR122 API, your source code must include a header file "acr122.h".
 * @code
 * #include <stdio.h>
 * #include <acr122.h>
 *
 * int main(int argc, char *argv[])
 * {
 *     HANDLE hReader;
 *     DWORD ret;
 *
 *     // Open reader using COM1
 *     ret = ACR122_Open(TEXT("\\\\.\\COM1"), &hReader);
 *     if (ret != ERROR_SUCCESS)
 *     {
 *         printf("ACR122_Open failed with error 0x%08X\n", ret);
 *         return 1;
 *     }
 *
 *     // TODO: Place other API function calls here
 *     // ...
 *
 *     // Close reader
 *     ret = ACR122_Close(hReader);
 *     if (ret != ERROR_SUCCESS)
 *         printf("ACR122_Close failed with error 0x%08X\n", ret);
 *
 *     return 0;
 * }
 * @endcode
 *
 * @subsection OpeningReader Opening Reader
 *
 * On Windows, port name is specified as "\\.\COMx" where x is the port number.
 * e.g. "\\.\COM1" is the first serial port.
 *
 * On Linux, port name can be any file under directory "/dev". Stanard Linux
 * serial port driver creates the device file as "/dev/ttySx" where x is the
 * port number. e.g. "/dev/ttyS0" is the first serial port.
 *
 * ACR122_Open() will return a handle of reader in parameter if the port is
 * opened successfully.
 *
 * @subsection ClosingReader Closing Reader
 *
 * After finishing the operation, use ACR122_Close() to close the reader.
 *
 * @subsection UsingANSIandUnicodeFunctions Using ANSI and Unicode Functions
 *
 * If your source code defined <b>UNICODE</b> macro, the following functions
 * will be mapped to Unicode version.
 *
 * - ACR122_Open() -> ACR122_OpenW()
 * - ACR122_GetFirmwareVersion() -> ACR122_GetFirmwareVersionW()
 * - ACR122_DisplayLcdMessage() -> ACR122_DisplayLcdMessageW()
 * - ACR122_DisplayLcdMessageEx() -> ACR122_DisplayLcdMessageExW()
 * - ACR122_DisplayLcdMessageGB() -> ACR122_DisplayLcdMessageGBW()
 *
 * Otherwise, it will be mapped to ANSI version.
 *
 * - ACR122_Open() -> ACR122_OpenA()
 * - ACR122_GetFirmwareVersion() -> ACR122_GetFirmwareVersionA()
 * - ACR122_DisplayLcdMessage() -> ACR122_DisplayLcdMessageA()
 * - ACR122_DisplayLcdMessageEx() -> ACR122_DisplayLcdMessageExA()
 * - ACR122_DisplayLcdMessageGB() -> ACR122_DisplayLcdMessageGBA()
 */

#ifndef ACR122_H
#define ACR122_H

#include <windows.h>

// Error codes
#define ACR122_ERROR_NO_MORE_HANDLES    ((DWORD) 0x20000001L)   ///< There are no more handles.
#define ACR122_ERROR_UNKNOWN_STATUS     ((DWORD) 0x20000002L)   ///< The status of response is unknown or not returned.
#define ACR122_ERROR_OPERATION_FAILURE  ((DWORD) 0x20000003L)   ///< The operation failed.
#define ACR122_ERROR_OPERATION_TIMEOUT  ((DWORD) 0x20000004L)   ///< The operation timed out.
#define ACR122_ERROR_INVALID_CHECKSUM   ((DWORD) 0x20000005L)   ///< The checksum of response is invalid.
#define ACR122_ERROR_INVALID_PARAMETER  ((DWORD) 0x20000006L)   ///< The command is invalid.

// LED States
#define ACR122_LED_STATE_OFF            0   ///< Turn on LED.
#define ACR122_LED_STATE_ON             1   ///< Turn off LED.

// Buzzer Mode
#define ACR122_BUZZER_MODE_OFF          0x00    ///< The buzzer will not turn on.
#define ACR122_BUZZER_MODE_ON_T1        0x01    ///< The buzzer will turn on during T1 duration.
#define ACR122_BUZZER_MODE_ON_T2        0x02    ///< The buzzer will turn on during T2 duration.

// LED Control
typedef struct _ACR122_LED_CONTROL {
    /**
     * Final state. Possible values are ACR122_LED_STATE_OFF and
     * ACR122_LED_STATE_ON.
     */
    DWORD finalState;
    
    /**
     * Enable update. Set to TRUE to update the state. Otherwise, set to FALSE
     * to keep the state unchanged.
     */
    BOOL updateEnabled;
    
    /**
     * Initial blinking state. Possible values are ACR122_LED_STATE_OFF and
     * ACR122_LED_STATE_ON.
     */
    DWORD initialBlinkingState;
    
    /**
     * Enable blink. Set to TRUE to enable blink. Otherwise, set to FALSE.
     */
    BOOL blinkEnabled;          
} ACR122_LED_CONTROL, *PACR122_LED_CONTROL;

/**
 * @struct _ACR122_LED_CONTROL
 * LED Control. This data structure is used in ACR122_SetLedStatesWithBeep()
 * function.
 */

/**
 * @typedef ACR122_LED_CONTROL
 * Create a type name for _ACR122_LED_CONTROL.
 */

/**
 * @typedef PACR122_LED_CONTROL
 * Create a type name for pointer to _ACR122_LED_CONTROL data structure.
 */

// Timeouts
typedef struct _ACR122_TIMEOUTS {
    /**
     * Status timeout in milliseconds. Default is 2000 ms.
     */
    DWORD statusTimeout;

    /**
     * Number of status retries. Default is 1.
     */
    DWORD numStatusRetries;

    /**
     * Response timeout in milliseconds. Default is 10000 ms.
     */
    DWORD responseTimeout;

    /**
     * Number of response retries. Default is 1.
     */
    DWORD numResponseRetries;
} ACR122_TIMEOUTS, *PACR122_TIMEOUTS;

/**
 * @struct _ACR122_TIMEOUTS
 * Timeouts. This data structure is used in ACR122_GetTimeouts() and
 * ACR122_SetTimeouts() function.
 */

/**
 * @typedef ACR122_TIMEOUTS
 * Create a type name for _ACR122_TIMEOUTS.
 */

/**
 * @typedef PACR122_TIMEOUTS
 * Create a type name for pointer to _ACR122_TIMEOUTS data structure.
 */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @addtogroup Reader
 */

/*@{*/

/**
 * Open reader (ANSI). The ACR122_OpenA() function opens a reader and returns a
 * handle value as reference.
 * @param[in]  portName Port name.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>Port name is specified as "\\.\COMx" where x is the
 * port number. e.g. "\\.\COM1" is the first serial port.</td></tr>
 * <tr><td>Linux</td><td>Port name can be any file under directory "/dev".
 * Stanard Linux serial port driver creates the device file as "/dev/ttySx"
 * where x is the port number. e.g. "/dev/ttyS0" is the first serial port.</td>
 * </tr>
 * </table><br>
 * @param[out] phReader Pointer to the HANDLE variable.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Close
 *
 * @code
 * HANDLE hReader;
 * DWORD ret;
 *
 * // Open reader using COM1
 * ret = ACR122_Open(TEXT("\\\\.\\COM1"), &hReader);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_Open failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_OpenA(LPCSTR portName, LPHANDLE phReader);

/**
 * Open reader (Unicode). The ACR122_OpenW() function opens a reader and returns a
 * handle value as reference.
 * @param[in]  portName Port name.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>Port name is specified as "\\.\COMx" where x is the
 * port number. e.g. "\\.\COM1" is the first serial port.</td></tr>
 * <tr><td>Linux</td><td>Port name can be any file under directory "/dev".
 * Stanard Linux serial port driver creates the device file as "/dev/ttySx"
 * where x is the port number. e.g. "/dev/ttyS0" is the first serial port.</td>
 * </tr>
 * </table><br>
 * @param[out] phReader Pointer to the HANDLE variable.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Close
 *
 * @code
 * HANDLE hReader;
 * DWORD ret;
 *
 * // Open reader using COM1
 * ret = ACR122_Open(TEXT("\\\\.\\COM1"), &hReader);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_Open failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_OpenW(LPCWSTR portName, LPHANDLE phReader);

/**
 * @def ACR122_Open
 * ACR122_Open will be mapped to ACR122_OpenW() function if UNICODE is defined.
 * Otherwise, it will be mapped to ACR122_OpenA() function.
 */
#ifdef UNICODE
#define ACR122_Open  ACR122_OpenW
#else
#define ACR122_Open  ACR122_OpenA
#endif

/**
 * Close reader. The ACR122_Close() function closes the reader and releases the
 * resources.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Close reader
 * ret = ACR122_Close(hReader);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_Close failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_Close(HANDLE hReader);

/**
 * Get number of slots. The ACR122_GetNumSlots() function retrieves the number
 * of slots on reader.
 * @param[in]  hReader   A reference value returned from ACR122_Open() function.
 * @param[out] pNumSlots Pointer to a DWORD variable in which the number of slots is returned.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD numSlots;
 * DWORD ret;
 *
 * // Get number of slots
 * ret = ACR122_GetNumSlots(hReader, &numSlots);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_GetNumSlots failed with error 0x%08X\n", ret);
 * @endcode
 */ 
DWORD WINAPI ACR122_GetNumSlots(HANDLE hReader, LPDWORD pNumSlots);

/**
 * Get baud rate. The ACR122_GetBaudRate() function retrieves the baud rate of
 * reader.
 * @param[in]  hReader   A reference value returned from ACR122_Open() function.
 * @param[out] pBaudRate Pointer to a DWORD variable in which the baud rate is returned.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_SetBaudRate
 *
 * @code
 * DWORD baudRate;
 * DWORD ret;
 *
 * // Get baud rate
 * ret = ACR122_GetBaudRate(hReader, &baudRate);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_GetBaudRate failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_GetBaudRate(HANDLE hReader, LPDWORD pBaudRate);

/**
 * Set baud rate. The ACR122_SetBaudRate() function sets the baud rate of
 * reader. Actully, the reader supports 9600 bps and 115200 bps.
 * @param[in] hReader  A reference value returned from ACR122_Open() function.
 * @param[in] baudRate Baud rate must be 9600 bps or 115200 bps.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_GetBaudRate
 *
 * @code
 * DWORD ret;
 *
 * // Set baud rate to 115200 bps
 * ret = ACR122_SetBaudRate(hReader, 115200);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_SetBaudRate failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_SetBaudRate(HANDLE hReader, DWORD baudRate);

/**
 * Get timeouts. The ACR122_GetTimeouts() function retrieves the timeout
 * parameters for status and response operations on the reader.
 * @param[in]  hReader   A reference value returned from ACR122_Open() function.
 * @param[out] pTimeouts Pointer to a ACR122_TIMEOUTS structure in which the timeout information is returned.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_SetTimeouts
 *
 * @code
 * ACR122_TIMEOUTS timeouts;
 * DWORD ret;
 *
 * // Get timeouts
 * ret = ACR122_GetTimeouts(hReader, &timeouts);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_GetTimeouts failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_GetTimeouts(HANDLE hReader, PACR122_TIMEOUTS pTimeouts);

/**
 * Set timeouts. The ACR122_SetTimeouts() function sets the timeout parameters
 * for status and response operations on the reader.
 * @param[in] hReader   A reference value returned from ACR122_Open() function.
 * @param[in] pTimeouts Pointer to a ACR122_TIMEOUTS structure that contains the new timeout values.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_GetTimeouts
 *
 * @code
 * ACR122_TIMEOUTS timeouts;
 * DWORD ret;
 *
 * // Get timeouts
 * // ...
 *
 * // Modify status timeout to 100 ms
 * timeouts.statusTimeout = 100;
 *
 * // Set timeouts
 * ret = ACR122_SetTimeouts(hReader, &timeouts);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_SetTimeouts failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_SetTimeouts(HANDLE hReader, const PACR122_TIMEOUTS pTimeouts);

/**
 * Get firmware version (ANSI). The ACR122_GetFirmwareVersionA() function
 * retrieves the firmware version in ANSI string of the slot.
 * @param[in]     hReader             A reference value returned from ACR122_Open() function.
 * @param[in]     slotNum             Slot number.
 * @param[out]    firmwareVersion     A pointer to the buffer that receives the firmware version returned from the reader.
 * @param[in,out] pFirmwareVersionLen The length in number of characters of the firmwareVersion parameter and receives the actual number of characters including last null character received from the reader.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * TCHAR firmwareVersion[20];
 * DWORD firmwareVersionLen;
 * DWORD ret;
 *
 * // Get firmware version
 * firmwareVersionLen = sizeof(firmwareVersion) / sizeof(TCHAR);
 * ret = ACR122_GetFirmwareVersion(hReader, firmwareVersion, &firmwareVersionLen);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_GetFirmwareVersion failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_GetFirmwareVersionA(HANDLE hReader, DWORD slotNum, LPSTR firmwareVersion, LPDWORD pFirmwareVersionLen);

/**
 * Get firmware version (Unicode). The ACR122_GetFirmwareVersionW() function
 * retrieves the firmware version in Unicode string of the slot.
 * @param[in]     hReader             A reference value returned from ACR122_Open() function.
 * @param[in]     slotNum             Slot number.
 * @param[out]    firmwareVersion     A pointer to the buffer that receives the firmware version returned from the reader.
 * @param[in,out] pFirmwareVersionLen The length in number of characters of the firmwareVersion parameter and receives the actual number of characters including last null character received from the reader.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * TCHAR firmwareVersion[20];
 * DWORD firmwareVersionLen;
 * DWORD ret;
 *
 * // Get firmware version
 * firmwareVersionLen = sizeof(firmwareVersion) / sizeof(TCHAR);
 * ret = ACR122_GetFirmwareVersion(hReader, firmwareVersion, &firmwareVersionLen);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_GetFirmwareVersion failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_GetFirmwareVersionW(HANDLE hReader, DWORD slotNum, LPWSTR firmwareVersion, LPDWORD pFirmwareVersionLen);

/**
 * @def ACR122_GetFirmwareVersion
 * ACR122_GetFirmwareVersion will be mapped to ACR122_GetFirmwareVersionW()
 * function if UNICODE is defined. Otherwise, it will be mapped to
 * ACR122_GetFirmwareVersionA() function.
 */
#ifdef UNICODE
#define ACR122_GetFirmwareVersion  ACR122_GetFirmwareVersionW
#else
#define ACR122_GetFirmwareVersion  ACR122_GetFirmwareVersionA
#endif

/*@}*/

/**
 * @addtogroup LCD
 * These functions are for ACR122L reader only.
 */

/*@{*/

/**
 * Display LCD message (ANSI). The ACR122_DisplayLcdMessageA() function
 * displays LCD message on the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @param[in] row     Row number must be from 0 to 1.
 * @param[in] col     Column number must be from 0 to 15.
 * @param[in] message Message for display. The length of message must be less than or equal to 16 characters.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Display text "Hello" at row 0 and column 0
 * ret = ACR122_DisplayLcdMessage(hReader, 0, 0, TEXT("Hello"));
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DisplayLcdMessage failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DisplayLcdMessageA(HANDLE hReader, DWORD row, DWORD col, LPCSTR message);

/**
 * Display LCD message (Unicode). The ACR122_DisplayLcdMessageW() function
 * displays LCD message on the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @param[in] row     Row number must be from 0 to 1.
 * @param[in] col     Column number must be from 0 to 15.
 * @param[in] message Message for display. The length of message must be less than or equal to 16 characters.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Display text "Hello" at row 0 and column 0
 * ret = ACR122_DisplayLcdMessage(hReader, 0, 0, TEXT("Hello"));
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DisplayLcdMessage failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DisplayLcdMessageW(HANDLE hReader, DWORD row, DWORD col, LPCWSTR message);

/**
 * @def ACR122_DisplayLcdMessage
 * ACR122_DisplayLcdMessage will be mapped to ACR122_DisplayLcdMessageW()
 * function if UNICODE is defined. Otherwise, it will be mapped to
 * ACR122_DisplayLcdMessageA() function.
 */
#ifdef UNICODE
#define ACR122_DisplayLcdMessage   ACR122_DisplayLcdMessageW
#else
#define ACR122_DisplayLcdMessage   ACR122_DisplayLcdMessageA
#endif

/**
 * Display LCD message (ANSI). The ACR122_DisplayLcdMessageExA() function
 * displays LCD message on the reader using one of the font tables such as
 * Latin, Cyrillic and Japanese with normal or bold font.
 * @param[in] hReader    A reference value returned from ACR122_Open() function.
 * @param[in] row        Row number. The value depends on which font table is selected and it must be from 0 to <i>Number of Rows</i> - 1.
 * <table>
 * <tr><td><b>Font Table</b></td><td><b>Number of Rows</b></td></tr>
 * <tr><td>Latin</td><td>2</td></tr>
 * <tr><td>Cyrillic</td><td>2</td></tr>
 * <tr><td>Japanese</td><td>4</td></tr>
 * </table><br>
 * @param[in] col        Column number must be from 0 to 15.
 * @param[in] tableIndex Select which font table to be used.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>0</td><td>Latin</td></tr>
 * <tr><td>1</td><td>Cyrillic</td></tr>
 * <tr><td>2</td><td>Japanese</td></tr>
 * </table><br>
 * @param[in] boldMode   Bold mode.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>0</td><td>Normal</td></tr>
 * <tr><td>1</td><td>Bold</td></tr>
 * </table><br>
 * @param[in] message    Message for display. The length of message must be less than or equal to 16 characters.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Display text "Hello" at row 0 and column 0 using Latin font table with normal font
 * ret = DisplayLcdMessageEx(hReader, 0, 0, 0, 0, TEXT("Hello"));
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DisplayLcdMessageEx failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DisplayLcdMessageExA(HANDLE hReader, DWORD row, DWORD col, DWORD tableIndex, DWORD boldMode, LPCSTR message);

/**
 * Display LCD message (Unicode). The ACR122_DisplayLcdMessageExW() function
 * displays LCD message on the reader using one of the font tables such as
 * Latin, Cyrillic and Japanese with normal or bold font.
 * @param[in] hReader    A reference value returned from ACR122_Open() function.
 * @param[in] row        Row number. The value depends on which font table is selected and it must be from 0 to <i>Number of Rows</i> - 1.
 * <table>
 * <tr><td><b>Font Table</b></td><td><b>Number of Rows</b></td></tr>
 * <tr><td>Latin</td><td>2</td></tr>
 * <tr><td>Cyrillic</td><td>2</td></tr>
 * <tr><td>Japanese</td><td>4</td></tr>
 * </table><br>
 * @param[in] col        Column number must be from 0 to 15.
 * @param[in] tableIndex Select which font table to be used.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>0</td><td>Latin</td></tr>
 * <tr><td>1</td><td>Cyrillic</td></tr>
 * <tr><td>2</td><td>Japanese</td></tr>
 * </table><br>
 * @param[in] boldMode   Bold mode.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>0</td><td>Normal</td></tr>
 * <tr><td>1</td><td>Bold</td></tr>
 * </table><br>
 * @param[in] message    Message for display. The length of message must be less than or equal to 16 characters.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Display text "Hello" at row 0 and column 0 using Latin font table with normal font
 * ret = DisplayLcdMessageEx(hReader, 0, 0, 0, 0, TEXT("Hello"));
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DisplayLcdMessageEx failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DisplayLcdMessageExW(HANDLE hReader, DWORD row, DWORD col, DWORD tableIndex, DWORD boldMode, LPCWSTR message);

/**
 * @def ACR122_DisplayLcdMessageEx
 * ACR122_DisplayLcdMessageEx will be mapped to ACR122_DisplayLcdMessageExW()
 * function if UNICODE is defined. Otherwise, it will be mapped to
 * ACR122_DisplayLcdMessageExA() function.
 */
#ifdef UNICODE
#define ACR122_DisplayLcdMessageEx   ACR122_DisplayLcdMessageExW
#else
#define ACR122_DisplayLcdMessageEx   ACR122_DisplayLcdMessageExA
#endif

/**
 * Display LCD message (ANSI). The ACR122_DisplayLcdMessageGBA() function
 * displays LCD message on the reader using Simplified Chinese font.
 * @param[in] hReader  A reference value returned from ACR122_Open() function.
 * @param[in] row      Row number must be from 0 to 1.
 * @param[in] col      Column number must be from 0 to 7.
 * @param[in] boldMode Bold mode.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>0</td><td>Normal</td></tr>
 * <tr><td>1</td><td>Bold</td></tr>
 * </table><br>
 * @param[in] message  Message for display. The length of message must be less than or equal to 8 characters.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Display text "欢迎" at row 0 and column 0 using normal font
 * ret = ACR122_DisplayLcdMessageGB(hReader, 0, 0, 0, TEXT("欢迎"));
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DisplayLcdMessageGB failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DisplayLcdMessageGBA(HANDLE hReader, DWORD row, DWORD col, DWORD boldMode, LPCSTR message);

/**
 * Display LCD message (Unicode). The ACR122_DisplayLcdMessageGBW() function
 * displays LCD message on the reader using Simplified Chinese font.
 * @param[in] hReader  A reference value returned from ACR122_Open() function.
 * @param[in] row      Row number must be from 0 to 1.
 * @param[in] col      Column number must be from 0 to 7.
 * @param[in] boldMode Bold mode.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>0</td><td>Normal</td></tr>
 * <tr><td>1</td><td>Bold</td></tr>
 * </table><br>
 * @param[in] message  Message for display. The length of message must be less than or equal to 8 characters.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Display text "欢迎" at row 0 and column 0 using normal font
 * ret = ACR122_DisplayLcdMessageGB(hReader, 0, 0, 0, TEXT("欢迎"));
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DisplayLcdMessageGB failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DisplayLcdMessageGBW(HANDLE hReader, DWORD row, DWORD col, DWORD boldMode, LPCWSTR message);

/**
 * @def ACR122_DisplayLcdMessageGB
 * ACR122_DisplayLcdMessageGB will be mapped to ACR122_DisplayLcdMessageGBW()
 * function if UNICODE is defined. Otherwise, it will be mapped to
 * ACR122_DisplayLcdMessageGBA() function.
 */
#ifdef UNICODE
#define ACR122_DisplayLcdMessageGB   ACR122_DisplayLcdMessageGBW
#else
#define ACR122_DisplayLcdMessageGB   ACR122_DisplayLcdMessageGBA
#endif

/**
 * Draw pixels on LCD. The ACR122_DrawLcd() function draws the pixels on LCD of
 * the reader.
 * @param[in] hReader        A reference value returned from ACR122_Open() function.
 * @param[in] lineIndex      Line index must be from 0 to 31.
 * @param[in] pixelBuffer    Buffer with pixel data to be displayed.
 * @param[in] pixelBufferLen Length of pixel buffer. The length must be less than or equal to 128 bytes.
 * @return Error code.  
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * BYTE buffer[] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
 * DWORD ret;
 *
 * // Draw a line at row 0
 * ret = ACR122_DrawLcd(hReader, 0, buffer, sizeof(buffer));
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DrawLcd failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DrawLcd(HANDLE hReader, DWORD lineIndex, const LPBYTE pixelBuffer, DWORD pixelBufferLen);

/**
 * Start LCD scrolling. The ACR122_StartLcdScrolling() function starts the
 * scrolling on LCD of the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @param[in] row     Row number must be 0.
 * @param[in] col     Column number must be 0.
 * @param[in] HRange  Horizontal range (Unit: 8 pixels). The value must be 15.
 * @param[in] VRange  Vertical range (Unit: 1 pixel). The value must be 31.
 * @param[in] speed   Speed.<br>
 *                    Bit0 - Bit3: Number of pixels in pre-scrolling.<br>
 *                    Bit4 - Bit7: Scrolling period.
 * <table>
 * <tr><td><b>Value (Bit4 - Bit7)</b></td><td><b>Scrolling Period</b></td></tr>
 * <tr><td>0</td><td>1 Unit</td></tr>
 * <tr><td>1</td><td>3 Units</td></tr>
 * <tr><td>2</td><td>5 Units</td></tr>
 * <tr><td>3</td><td>7 Units</td></tr>
 * <tr><td>4</td><td>17 Units</td></tr>
 * <tr><td>5</td><td>19 Units</td></tr>
 * <tr><td>6</td><td>21 Units</td></tr>
 * <tr><td>7</td><td>23 Units</td></tr>
 * <tr><td>8</td><td>129 Units</td></tr>
 * <tr><td>9</td><td>131 Units</td></tr>
 * <tr><td>10</td><td>133 Units</td></tr>
 * <tr><td>11</td><td>135 Units</td></tr>
 * <tr><td>12</td><td>145 Units</td></tr>
 * <tr><td>13</td><td>147 Units</td></tr>
 * <tr><td>14</td><td>149 Units</td></tr>
 * <tr><td>15</td><td>151 Units</td></tr>
 * </table><br>
 * @param[in] direction Direction.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>0</td><td>From left to right</td></tr>
 * <tr><td>1</td><td>From right to left</td></tr>
 * <tr><td>2</td><td>From top to bottom</td></tr>
 * <tr><td>3</td><td>From bottom to top</td></tr>
 * </table><br>
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_StopLcdScrolling, ACR122_PauseLcdScrolling
 *
 * @code
 * DWORD ret;
 *
 * // Start LCD scrolling at row 0 and column 0 from left to right
 * ret = ACR122_StartLcdScrolling(hReader, 0, 0, 15, 31, 0xF5, 0);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_StartLcdScrolling failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_StartLcdScrolling(HANDLE hReader, DWORD row, DWORD col, DWORD HRange, DWORD VRange, DWORD speed, DWORD direction);

/**
 * Stop LCD scrolling. The ACR122_LCDStopScrolling() function stops the
 * scrolling on LCD of the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_StartLcdScrolling, ACR122_PauseLcdScrolling
 *
 * @code
 * DWORD ret;
 *
 * // Stop LCD scrolling
 * ret = ACR122_StopLcdScrolling(hReader);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_StopLcdScrolling failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_StopLcdScrolling(HANDLE hReader);

/**
 * Pause LCD scrolling. The ACR122_LCDPauseScrolling() function pauses the
 * scrolling on LCD of the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_StartLcdScrolling, ACR122_StopLcdScrolling
 *
 * @code
 * DWORD ret;
 *
 * // Pause LCD scrolling
 * ret = ACR122_PauseLcdScrolling(hReader);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_PauseLcdScrolling failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_PauseLcdScrolling(HANDLE hReader);

/**
 * Clear LCD. The ACR122_ClearLcd() function clears the LCD of the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Clear LCD
 * ret = ACR122_ClearLcd(hReader);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_ClearLcd failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_ClearLcd(HANDLE hReader);

/**
 * Enable LCD backlight. The ACR122_EnableLcdBacklight() function enables or
 * disables the LCD backlight of the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @param[in] enabled Set to TRUE to enable backlight. Otherwise, set to FALSE.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Enable LCD backlight
 * ret = ACR122_EnableLcdBacklight(hReader, TRUE);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_EnableLcdBacklight failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_EnableLcdBacklight(HANDLE hReader, BOOL enabled);

/**
 * Set LCD contrast. The ACR122_SetLcdContrast() function sets the LCD oontrast
 * level of the reader.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @param[in] level   Contrast level. The value must be from 0 to 15.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Set LCD contrast to 10
 * ret = ACR122_SetLcdContrast(hReader, 10);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_SetLcdContrast failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_SetLcdContrast(HANDLE hReader, DWORD level);

/*@}*/

/**
 * @addtogroup LED
 */

/*@{*/

/**
 * Enable LED. The ACR122_EnableLed() function enables or disables LED control
 * to the application. By default, LED is controlled by the firmware. Before
 * calling ACR122_SetLedStatesWithBeep() and ACR122_SetLedStates(), the
 * application needs to call this function in order to control the LED. This
 * function is for ACR122L reader only.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @param[in] enabled Set to TRUE to enable LED. Otherwise, set to FALSE.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_SetLedStatesWithBeep, ACR122_SetLedStates
 *
 * @code
 * DWORD ret;
 *
 * // Enable LED
 * ret = ACR122_EnableLed(hReader, TRUE);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_EnableLed failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_EnableLed(HANDLE hReader, BOOL enabled);

/**
 * Set LED states with beep. The ACR122_SetLedStatesWithBeep() controls LED0,
 * LED1 and buzzer operation on the reader.
 * @param[in] hReader     A reference value returned from ACR122_Open() function.
 * @param[in] controls    A pointer to the array of ACR122_LED_CONTROL data structure.
 * @param[in] numControls Number of controls must be 2.
 * @param[in] t1          T1 in milliseconds. The value must be from 0 to 25500.
 * @param[in] t2          T2 in milliseconds. The value must be from 0 to 25500.
 * @param[in] numTimes    Number of times. The values must be from 0 to 255.
 * @param[in] buzzerMode  A bitmask of buzzer mode. Possible values may be combined with the OR operation.
 * <table>
 * <tr><td><b>Value</b></td><td><b>Meaning</b></td></tr>
 * <tr><td>ACR122_BUZZER_MODE_OFF</td><td>The buzzer will not turn on.</td></tr>
 * <tr><td>ACR122_BUZZER_MODE_ON_T1</td><td>The buzzer will turn on during T1 duration.</td></tr>
 * <tr><td>ACR122_BUZZER_MODE_ON_T2</td><td>The buzzer will turn on during T2 duration.</td></tr>
 * </table><br>
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_EnableLed
 *
 * @code
 * ACR122_LED_CONTROL controls[2];
 * DWORD ret;
 *
 * // Set LED0 to ON
 * controls[0].finalState = ACR122_LED_STATE_ON;
 * controls[0].updateEnabled = TRUE;
 * controls[0].initialBlinkingState = ACR122_LED_STATE_OFF;
 * controls[0].blinkEnabled = FALSE;
 *
 * // Set LED1 to blink
 * controls[1].finalState = ACR122_LED_STATE_OFF;
 * controls[1].updateEnabled = FALSE;
 * controls[1].initialBlinkingState = ACR122_LED_STATE_OFF;
 * controls[1].blinkEnabled = TRUE;
 *
 * // Beep on T1 where T1 and T2 are equal to 100 ms
 * ret = ACR122_SetLedStatesWithBeep(hReader, controls, 2, 100, 100, ACR122_BUZZER_MODE_ON_T1);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_SetLedStatesWithBeep failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_SetLedStatesWithBeep(HANDLE hReader, const PACR122_LED_CONTROL controls, DWORD numControls, DWORD t1, DWORD t2, DWORD numTimes, DWORD buzzerMode);

/**
 * Set LED states. The ACR122_SetLedStates() function turns on or off the LEDs
 * on the reader. LED0, LED1, LED2 and LED3 can be controlled. This function is
 * for ACR122L reader only.
 * @param[in] hReader   A reference value returned from ACR122_Open() function.
 * @param[in] states    A pointer to the array of states. Possible values are ACR122_LED_STATE_OFF and ACR122_LED_STATE_ON.
 * @param[in] numStates Number of states must be 4.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open, ACR122_EnableLed
 *
 * @code
 * DWORD states[4];
 * DWORD ret;
 *
 * // Set LED0 and LED2 to OFF, LED1 and LED3 to ON
 * states[0] = ACR122_LED_STATE_OFF;
 * states[1] = ACR122_LED_STATE_ON;
 * states[2] = ACR122_LED_STATE_OFF;
 * states[3] = ACR122_LED_STATE_ON;
 *
 * ret = ACR122_SetLedStates(hReader, states, 4);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_SetLedStates failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_SetLedStates(HANDLE hReader, const LPDWORD states, DWORD numStates);

/*@}*/

/**
 * @addtogroup Buzzer
 */

/*@{*/

/**
 * Beep. The ACR122_Beep() function controls the buzzer on the reader to
 * generate the beep sound and it does not return control to its caller until
 * the sound finishes.
 * @param[in] hReader           A reference value returned from ACR122_Open() function.
 * @param[in] buzzerOnDuration  Buzzer ON duration in milliseconds. The value must be from 0 to 25500.
 * @param[in] buzzerOffDuration Buzzer OFF duration in milliseconds. The value must be from 0 to 25500.
 * @param[in] numTimes          Number of times.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Beep (ON: 100ms, OFF: 100ms, 1 time)
 * ret = ACR122_Beep(hReader, 100, 100, 1);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_Beep failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_Beep(HANDLE hReader, DWORD buzzerOnDuration, DWORD buzzerOffDuration, DWORD numTimes);

/*@}*/

/**
 * @addtogroup Card
 */

/*@{*/

/**
 * Direct transmit PN532 and TAG command. The ACR122_DirectTransmit() function
 * sends PN532 and TAG command and receives response from the reader.
 * @param[in]     hReader        A reference value returned from ACR122_Open() function.
 * @param[in]     sendBuffer     A pointer to the actual data to be written to the card.
 * @param[in]     sendBufferLen  The length in number of bytes of the sendBuffer parameter.
 * @param[out]    recvBuffer     A pointer to any data returned from the card.
 * @param[in,out] pRecvBufferLen The length in number of bytes of the recvBuffer parameter and receives the actual number of bytes received from the card.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * BYTE command[] = { 0xD4, 0x4A, 0x01, 0x00 }; // Poll Type A card
 * DWORD commandLen = sizeof(command);
 * BYTE response[300];
 * DWORD responseLen = sizeof(response);
 * DWORD ret;
 *
 * ret = ACR122_DirectTransmit(hReader, command, commandLen, response, &responseLen);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_DirectTransmit failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_DirectTransmit(HANDLE hReader, const LPBYTE sendBuffer, DWORD sendBufferLen, LPBYTE recvBuffer, LPDWORD pRecvBufferLen);

/**
 * Power on ICC in slot. The ACR122_PowerOnIcc function powers on the card in
 * the slot and returns the ATR string from the card.
 * @param[in]     hReader A reference value returned from ACR122_Open() function.
 * @param[in]     slotNum Slot number.
 * @param[out]    atr     A pointer to the buffer that receives the ATR string returned from the card.
 * @param[in,out] pAtrLen The length in number of bytes of the atr parameter and receives the actual number of bytes received from the card.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * BYTE atr[64];
 * DWORD atrLen = sizeof(atr);
 * DWORD ret;
 *
 * // Power on slot 0
 * ret = ACR122_PowerOnIcc(hReader, 0, atr, &atrLen);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_PowerOnIcc failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_PowerOnIcc(HANDLE hReader, DWORD slotNum, LPBYTE atr, LPDWORD pAtrLen);

/**
 * Power off ICC in slot. The ACR122_PowerOffIcc() function powers off the card
 * in the slot.
 * @param[in] hReader A reference value returned from ACR122_Open() function.
 * @param[in] slotNum Slot number.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * DWORD ret;
 *
 * // Power off slot 0
 * ret = ACR122_PowerOffIcc(hReader, 0);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_PowerOffIcc failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_PowerOffIcc(HANDLE hReader, DWORD slotNum);

/**
 * Exchange APDU. The ACR122_ExchangeApdu() function sends APDU command and
 * receives APDU response from the card.
 * @param[in]     hReader        A reference value returned from ACR122_Open() function.
 * @param[in]     slotNum        Slot number.
 * @param[in]     sendBuffer     A pointer to the actual data to be written to the card.
 * @param[in]     sendBufferLen  The length in number of bytes of the sendBuffer parameter.
 * @param[out]    recvBuffer     A pointer to any data returned from the card.
 * @param[in,out] pRecvBufferLen The length in number of bytes of the recvBuffer parameter and receives the actual number of bytes received from the card.
 * @return Error code.
 * @retval ERROR_SUCCESS The operation completed successfully.
 * @retval Failure       An error code.
 * <table>
 * <tr><td><b>Platform</b></td><td><b>Description</b></td></tr>
 * <tr><td>Windows</td><td>See error codes from Windows API and ACR122 error codes.</td></tr>
 * <tr><td>Linux</td><td>See error codes from GNU C library header file "errno.h" and ACR122 error codes.</td></tr>
 * </table><br>
 * @see ACR122_Open
 *
 * @code
 * BYTE command[] = { 0x80, 0x84, 0x00, 0x00, 0x08 };
 * DWORD commandLen = sizeof(command);
 * BYTE response[300];
 * DWORD responseLen = sizeof(response);
 * DWORD ret;
 *
 * // Exchange APDU on slot 0
 * ret = ACR122_ExchangeApdu(hReader, 0, command, commandLen, response, &responseLen);
 * if (ret != ERROR_SUCCESS)
 *     printf("ACR122_ExchangeApdu failed with error 0x%08X\n", ret);
 * @endcode
 */
DWORD WINAPI ACR122_ExchangeApdu(HANDLE hReader, DWORD slotNum, const LPBYTE sendBuffer, DWORD sendBufferLen, LPBYTE recvBuffer, LPDWORD pRecvBufferLen);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif
